package com.bazaarvoice.hybris.xml;


import com.bazaarvoice.hybris.exception.BazaarvoiceTransactionsExportException;
import com.bazaarvoice.hybris.utils.BVExportXmlUtils;
import com.bazaarvoice.hybris.xml.tags.InteractionTag;
import com.bazaarvoice.hybris.xml.tags.PaymentTransactions;
import com.bazaarvoice.hybris.xml.tags.ProductTag;
import com.bazaarvoice.hybris.xml.tags.TransactionProductTag;
import com.sun.xml.bind.marshaller.CharacterEscapeHandler;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.core.model.order.AbstractOrderModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.product.ProductModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static com.bazaarvoice.hybris.utils.BazaarVoiceUtils.ReplaceUnsupportedCharacters;

@Scope("prototype")
@Component
public class PaymentTransactionsXmlCreator {

    private static final Logger LOG = Logger.getLogger(PaymentTransactionsXmlCreator.class.getName());

    @Resource(name = "bvExportXmlUtils")
    private BVExportXmlUtils bvExportXmlUtils;

    public List<InteractionTag> createInteractionTagFromCollection(Collection<PaymentTransactionModel> transactionModels,
                                                                   CMSSiteModel cmsSiteModel) {
        List<InteractionTag> interactionTags = new ArrayList<>();
        transactionModels.forEach(transactionModel -> {
            AbstractOrderModel orderModel = transactionModel.getOrder();
            if (orderModel != null) {
                validateTransactionModel(transactionModel);
                if (orderModel.getSite().getUid().equals(cmsSiteModel.getUid())) {
                    InteractionTag interactionTag = createInteractionTag(transactionModel, cmsSiteModel);
                    interactionTags.add(interactionTag);
                }
            } else {
                LOG.info("Transaction with pk: [" + transactionModel.getPk() + "] does not contain an order model, hence "
                        + "can not be included into Transactions export");
            }
        });
        return interactionTags;
    }

    private void validateTransactionModel(PaymentTransactionModel transactionModel) {
        if (!(transactionModel.getOrder().getSite() instanceof CMSSiteModel)) {
            throw new BazaarvoiceTransactionsExportException("Site has to be an instance of CMSSiteModel");
        }

        if (transactionModel.getOrder().getUser() == null) {
            throw new BazaarvoiceTransactionsExportException("User can not be null");
        }

        if (!(transactionModel.getOrder().getUser() instanceof CustomerModel)) {
            throw new BazaarvoiceTransactionsExportException("User has to be an instance of CustomerModel");
        }
    }

    private InteractionTag createInteractionTag(PaymentTransactionModel transactionModel, CMSSiteModel cmsSiteModel) {
        InteractionTag interactionTag = new InteractionTag();
        String deploymentZone = cmsSiteModel.getBvConfig().getZone();
        String deploymentZoneWithoutSpaces = deploymentZone.replaceAll("\\s+","_");
        interactionTag.setDeploymentZone(deploymentZoneWithoutSpaces.toLowerCase());
        interactionTag.setLocale(cmsSiteModel.getLocale());
        interactionTag.setTransactionDate(transactionModel.getCreationtime());
        interactionTag.setUserName(transactionModel.getOrder().getUser().getName());

        CustomerModel customerModel = ((CustomerModel) transactionModel.getOrder().getUser());
        if (customerModel.getContactEmail() != null) {
            interactionTag.setEmailAddress(customerModel.getContactEmail());
        } else if (customerModel.getUid().contains("@")) {
            interactionTag.setEmailAddress(customerModel.getUid());
        }

        interactionTag.setProducts(createProductTagCollection(transactionModel.getOrder(), cmsSiteModel));

        return interactionTag;
    }

    private List<TransactionProductTag> createProductTagCollection(AbstractOrderModel orderModel, CMSSiteModel cmsSiteModel) {
        List<TransactionProductTag> productTags = new ArrayList<>();

        orderModel.getEntries().forEach(entry -> {
            TransactionProductTag transactionProductTag = new TransactionProductTag();

            ProductModel productModel = entry.getProduct();
            transactionProductTag.setExternalId(bvExportXmlUtils.getProductExternalId(productModel, cmsSiteModel));

            String imageUrlString = bvExportXmlUtils.getImageUrl(productModel, cmsSiteModel);
            transactionProductTag.setImageUrl((imageUrlString == null || imageUrlString.isEmpty()) ? null : imageUrlString);

            final String name = productModel.getName();
            transactionProductTag.setName(name == null || name.isEmpty() ? productModel.getCode() : name);

            transactionProductTag.setPrice(entry.getBasePrice().toString());

            productTags.add(transactionProductTag);
        });
        return productTags;
    }

    public void writeXmlToFile(final File file, final PaymentTransactions paymentTransactions) {

        LOG.debug("===========Marshalling Payment Transactions to file " + file.getAbsolutePath() + "===================");

        try {
            final JAXBContext jaxbContext = JAXBContext.newInstance(PaymentTransactions.class);
            final Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
            jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            jaxbMarshaller.setProperty(CharacterEscapeHandler.class.getName(),
                    new CharacterEscapeHandler() {
                        @Override
                        public void escape(char[] ac, int i, int j, boolean flag,
                                           Writer writer) throws IOException {
                            writer.write(ac, i, j);
                        }
                    });
            jaxbMarshaller.marshal(paymentTransactions, file);

        } catch (final JAXBException e) {
            LOG.error(e.getStackTrace());
        }
    }


}
